// -*-C++-*-
//
// File:      BinaryFormatter.cc
// Author:    Jorgo Bakker (jbakker@rssd.esa.int)
// Generated: Mon Jan 27 17:59:41 CET 2003
// Usage:     -
// Info:      -

#include "BinaryFormatter.h"
#include <stringutil.h> // ::fullbreakup()

#include <iostream>
#include <unistd.h> // ::read()/w:;write()

const char* _sep="\r";


BinaryFormatter::BinaryFormatter() 
  : _fdi(-1),_fdo(-1),_handler(0)
{
}

BinaryFormatter::~BinaryFormatter()
{
}

void BinaryFormatter::setInputChannel(int fd) {
  _fdi=fd;
}

void BinaryFormatter::setOutputChannel(int fd){
  _fdo=fd;
}

void BinaryFormatter::setDecodeListener(MessageHandler* handler) {
  _handler=handler;
}

void BinaryFormatter::message(const std::string& name,
			       const std::string& text)
{
  write(std::string("M")+_sep+name+_sep+text);
}

void BinaryFormatter::warning(const std::string& name,
			       const std::string& code, 
			       const std::string& text) {
  write(std::string("W")+_sep+name+_sep+text+_sep+code);
}

void BinaryFormatter::error(const std::string& name,
			     const std::string& code, 
			     const std::string& text) {
  write(std::string("E")+_sep+name+_sep+text+_sep+code);
}

void BinaryFormatter::fatal(const std::string& name,
			     const std::string& code, 
			     const std::string& text) {
  write(std::string("F")+_sep+name+_sep+text+_sep+code);
}

void BinaryFormatter::decode()
{
  if (!_handler) return; // TODO:

  // reads the channel, and figures out what to do
  int bytes=0;
  
  if ( sizeof(bytes) != ::read(_fdi, &bytes, sizeof(bytes)) ) {
    std::cerr << "BinaryFormatter::decode() ...?"
	      << std::endl;
    return;
  }

  char* data=new char[bytes];
  int bytesRead=::read(_fdi,data,bytes);
  if ( bytesRead == bytes ) {
    read(string(data,bytes));
  } else {
    std::cerr << "BinaryFormatter::decode() "
	      << bytesRead << "!=" << bytes
	      << std::endl;
  }

  delete [] data;
}

void BinaryFormatter::read(const std::string& data)
{
  if (data.empty()) return; // TODO: now what?
  
  /* tokens = {ID,NAME,TEXT,CODE}*/
  
  // break up in tokens, and return as unknown if not breakable
  std::vector<std::string> tokens=fullbreakup(data,_sep);
  int sz=tokens.size();
  if (sz < 3) return; // TODO: now what? _handler->unknown(data);

  /* possibly a message that we can recognize */
  const string& id  =tokens[0];
  const string& name=tokens[1];
  const string& text=tokens[2];
  if      (id == "M" && sz == 3) {
    _handler->message(name,text);
  }
  else if (sz == 4) {
    const string& code=tokens[3];
    if      (id == "W") _handler->warning(name,code,text);
    else if (id == "E") _handler->error(name,code,text);
    else if (id == "F") _handler->fatal(name,code,text);
    /*else  TODO: now what? _handler->unknown(data);*/
  }
  /* else {TODO: now what? _handler->unknown(data);}*/
}


void BinaryFormatter::write(const std::string& s)
{
  // TODO: real check for failure?
  int bytes=s.length();
  int written= ::write(_fdo, &bytes, sizeof(bytes));
  if ( written != sizeof(bytes) ) {
    std::cerr << "BinaryFormatter::write(" <<s
	      << ")... could not write header! "
	      << "expected=" << sizeof(bytes) << "!=" 
	      << written << "=written."
	      << std::endl;
    return;
  }

  written= ::write(_fdo,s.c_str(),bytes);
  if ( written != bytes ) {
    std::cerr << "BinaryFormatter::write(" <<s
	      << ")... could not write contents! "
	      << "expected=" << bytes << "!=" 
	      << written << "=written."
	      << std::endl;
  }

}
