/**-*-C++-*-*******************************************************************
 *
 *  MessageFormatter.h                          XMM Science Analysis Subsystem
 *
 *  Class to encode/decode various types of warning/error message, so that
 *  they may be passed via the "std::cerr" stream to the calling process.
 *
 *  (c) 1999 ESA  -  Written by Jon Brumfitt, Aurora
 *  (c) 2000 ESA  -  Modified by Jorgo Bakker, BSC
 *  (c) 2003 ESA  -  Refactored by Jorgo Bakker, BSC
 *
 *  $Id: MessageFormatter.h,v 1.3 2003/01/28 10:45:04 jbakker Exp $
 *
 *****************************************************************************/

#ifndef MessageFormatter_h
#define MessageFormatter_h

#include "MessageHandler.h"
#include <string>


// TODO: redocument refactoring of this class... above does not apply no more?
class MessageFormatter : public MessageHandler
{
public:
  MessageFormatter(){}
  virtual ~MessageFormatter() {}

  /*--------------------------------------------------------------------------
    encoding...
    -------------------------------------------------------------------------*/
  virtual void message(const std::string& name,
		       const std::string& text) =0;

  virtual void warning(const std::string& name,
		       const std::string& code, 
		       const std::string& text) =0;

  virtual void error(const std::string& name,
		     const std::string& code, 
		     const std::string& text) =0;

  virtual void fatal(const std::string& name,
		     const std::string& code, 
		     const std::string& text) =0;

  /*--------------------------------------------------------------------------
    decoding...
    -------------------------------------------------------------------------*/
  /**
     Decodes the data and passes it to the registered MessageHandler
  */
  virtual void decode() =0;

  /**
     The MessageHandler that listens to decode messages
   */
  virtual void setDecodeListener(MessageHandler* handler=0) =0;

private:
  // disabled:
  MessageFormatter(const MessageFormatter&);
  MessageFormatter& operator=(const MessageFormatter&);
};

#endif
