/**-*-C++-*-*******************************************************************
 *
 *  SimpleHandler.h                        XMM Science Analysis Subsystem
 *
 *  Simple implementation of the ErrorHandler interface.
 *
 *  (c) 1998  ESA  -  Written by Marco Beijersbergen
 *
 *  $Id: SimpleHandler.h,v 1.15 2003/01/28 10:45:04 jbakker Exp $
 *
 *****************************************************************************/

#ifndef SimpleHandler_h
#define SimpleHandler_h

#include "ErrorHandler.h"
#include "Msg.h"

class MessageFormatter;

class SimpleHandler : public ErrorHandler {

public:
  
  SimpleHandler();  
  virtual ~SimpleHandler();
  ErrorHandler& operator=(const ErrorHandler& h) {
    _name = h.name();
    _verbosity = h.verbosity();
    return *this;
  }

  void name(const std::string& name) {_name=name;}
  const std::string& name() const {return _name;}

  void verbosity(Verbosity level) {_verbosity = level;}
  Verbosity verbosity() const {return _verbosity;}
  Verbosity verbosity(Layer layer) const {return _verbosity - int(layer);}

  virtual void fatal(Code code, const std::string& text);  
  virtual void error(Code code, const std::string& text);  
  virtual void warning(Code code, const std::string& text);
  virtual void message(Layer layer, Verbosity level,const std::string& text);

  // simply pass on, and do nothing
  virtual void tryExternal(ExternRoutine routine) { routine(); }
protected:

  void stop();
  MessageFormatter*  _formatter;

private:  

  std::string        _name;
  int                _verbosity;
};

#endif
