// -*-C++-*-
//
// File:      SuppressHandler.cc
// Author:    Jorgo Bakker (jbakker@astro.estec.esa.nl)
// Generated: Mon Nov  6 10:05:12 MET 2000

#include "SuppressHandler.h"
#include <sstream>
#include <cstdlib> // getenv


SuppressHandler::SuppressHandler() : 
  _all(false),_number(-1)
{
  /* check the environment: if set, silence all warnings */
  
  if(getenv("SAS_SUPPRESS_WARNING") != 0) {
    std::string c;
    std::stringstream ss;
    ss << getenv("SAS_SUPPRESS_WARNING");
    while (ss >> c) ignore(c);
  }
}

SuppressHandler::~SuppressHandler()
{
}

void SuppressHandler::ignore(const Code& c)
{
  int counter=-1;
  {
    std::stringstream ss;
    ss << c;
    ss >> counter;
  }
  
  if      (c.empty() || counter == 0) _all=true;
  else if (counter > 0)               _number=counter;
  else                                _list[c]=1;
}

void SuppressHandler::allow (const Code& c)
{
  int counter=-1;
  {
    std::stringstream ss;
    ss << c;
    ss >> counter;
  }
  
  if (c.empty() || counter == 0) {
    _all=false;
    _list.clear();
  }
  else if (counter > 0) _number=counter;
  else                  _list.erase(c);
}

bool SuppressHandler::suppress(const Code& c)
{
  int number= ++_map[c];

  // ignore all?
  if (_all) return true;

  
  // ignore registered code?
  if (_list[c]) return true;


  // ignore after n times?
  if (_number > -1) return ( (number > _number)?true:false );

  // default
  return false;
}

std::string SuppressHandler::statistics()
{
  std::ostringstream ss;
  bool silenced=false;
  std::string s;

  for (std::map<Code,int>::iterator it=_map.begin();it != _map.end();it++) {
    int number=0;
    
    // count the number of times a warning is silenced
    if      (_all)               number=(*it).second;
    else if (_list[(*it).first]) number=(*it).second;
    else if (_number > 0)        number=(*it).second - _number;

    if ( number > 0 ) {
      ss << "\n   warning " << (*it).first
	 << " silently occurred " << number << " times";
      silenced=true;
    }
  }

  if (silenced) s=ss.str();
  return s;
}

void SuppressHandler::reset()
{
  _map.clear();
}

std::string SuppressHandler::options()
{
  std::ostringstream ss;
  
  if (_all) {
    ss << " -w";
  }
  else {
    // specify all codes that need to be silenced by default
    for (std::map<Code,int>::iterator it=_list.begin();it != _list.end();it++)
      if ((*it).second) ss << " -w " << (*it).first;
    
    // specify # warnings
    if (_number > -1) ss << " -w " << _number;
  }

  return ss.str();
}
