// -*-C++-*-
//
// File:      testChannelFormatter.cc
// Author:    Jorgo Bakker (jbakker@rssd.esa.int)
// Generated: Fri Jan 24 14:07:32 CET 2003
// Usage:     -
// Info:      -
#include "BinaryFormatter.h"
#include "AsciiFormatter.h"
#include <iostream>
#include <string>
#include <unistd.h> //exit(int),  int pipe(int filedes[2]), ssize_t read(int fd, void *buf, size_t count);
#include <cstdlib>

using namespace std;

class MyMessageReader : public MessageHandler
{
public:
  enum Type {Message,Warning,Error,Fatal};  

  Type   expect;
  string name;
  string text;
  string code; // 

public:

  MyMessageReader(){}
  ~MyMessageReader(){}

  std::string read() {
    int bytes=0;
    string result;
  
    if ( sizeof(bytes) != ::read(_channel, &bytes, sizeof(bytes)) ) {
      std::cerr << "MyMessageReader::read() no bytes?" << std::endl;
      return result;
    }

    char* data=new char[bytes];
    int bytesRead=::read(_channel,data,bytes);
    if ( bytesRead == bytes ) {
      result=string(data,bytes);
    } else {
      std::cerr << "MyMessageReader::read() " << bytesRead
		<< "!=" << bytes << std::endl;
    }
    return result;
  }

  void handle(MessageFormatter& formatter)
  {
    this->text="This is a piece of text";
    this->name="Name";
    this->code="Code";
    
    
    this->expect=MyMessageReader::Message;
    formatter.message(this->name,this->text);
    formatter.decode();
    
    this->expect=MyMessageReader::Warning;
    formatter.warning(this->name,this->code,this->text);
    formatter.decode();
    
    this->expect=MyMessageReader::Error;
    formatter.error(this->name,this->code,this->text);
    formatter.decode();
    
    this->expect=MyMessageReader::Fatal;
    formatter.fatal(this->name,this->code,this->text);
    formatter.decode();
  }

  
  void message(const string& name,const string& text) {
    abortIfNot(MyMessageReader::Message);
    abortIfNot(name,this->name);
    abortIfNot(text,this->text);
  };

  void warning(const string& name,const string& code, const string& text) {
    abortIfNot(MyMessageReader::Warning,code,name,text);
  };
    
  void error(const string& name,const string& code, const string& text) {
    abortIfNot(MyMessageReader::Error,code,name,text);
  };
  void fatal(const string& name,const string& code, const string& text) {
    abortIfNot(MyMessageReader::Fatal,code,name,text);
  };

private:
  const string toPretty(Type t) 
  {
    switch (t){
    case MyMessageReader::Message:  return "Message";
    case MyMessageReader::Warning:  return "Warning";
    case MyMessageReader::Error:    return "Error";
    case MyMessageReader::Fatal:    return "Fatal";
    };
    return "???";
  }

  void stop() {
    cerr << "Expected {\n"
	 << toPretty(expect) << '\n'
	 << name  << '\n'
	 << code  << '\n'
	 << text  << '\n'
	 << '}'
	 << endl;
    exit(1);
  }

  void abortIfNot(Type t) {
    if (t != expect) {
      cerr << "Type " << toPretty(t) << " != " << toPretty(expect) << endl;
      stop();
    }
  }
  void abortIfNot(const string& lhs,const string& rhs) {
    if (lhs != rhs) {
      cerr << "Field " << lhs << " != " << rhs << endl;
      stop();
    }
  }
  void abortIfNot(Type t,
		  const string& code, const string& name, const string& text) {
    abortIfNot(t);
    abortIfNot(name,this->name);
    abortIfNot(text,this->text);
    abortIfNot(code,this->code);
  }

  int _channel;
};


int main(int , char**)
{
  int myPipe[2]; // [0]=read; [1]=write
  if (pipe(myPipe) != 0) {
    std::cerr << "Could not setup my pipe!" << std::endl;
    return 1;
  }

  MyMessageReader* h=new MyMessageReader;

  BinaryFormatter binary;
  binary.setInputChannel(myPipe[0]);
  binary.setOutputChannel(myPipe[1]);
  binary.setDecodeListener(h);
  h->handle(binary);

  AsciiFormatter ascii;
  h->handle(ascii);

  return 0;
}
